<!-- Macros and functions -->
<#--  Max levels of MIM nesting allowed  -->
<#assign maxLevelsDisplayed = 5/>
<#--  We store the visited mixtures in the composition list to avoid displaying cyclical MIMs  -->
<#assign visitedComponents = []/>

<#-- Returns a sequence of strings-->
<#function getTradeNames mixtureProductInformation>
  <#local tradeNames = [] />
  <#if mixtureProductInformation?has_content>
	<#list mixtureProductInformation as productInformation>
		<#if productInformation.ProductIdentifiers.TradeNames?has_content>
			<#list productInformation.ProductIdentifiers.TradeNames as tradeName>
				<#local tradeNames = tradeNames + [tradeName["TradeName"]] />
			</#list>
    	</#if>
	</#list>
  </#if>
  <#return tradeNames/>
</#function>

<#function getProductOtherNames mixtureProductInformation>
  <#local otherNames = [] />
  <#if mixtureProductInformation?has_content>
	<#list mixtureProductInformation as productInformation>
		<#if productInformation.ProductIdentifiers.OtherNames?has_content>
			<#list productInformation.ProductIdentifiers.OtherNames as otherName>
				<#local otherNames = otherNames + [otherName["OtherName"]] />
			</#list>
    	</#if>
	</#list>
  </#if>
  <#return otherNames/>
</#function>

<#function otherNames mixture>
	<#local result= []/>
	<#if (mixture.OtherNames?has_content)>
		<#list mixture.OtherNames as name>
            <#local type>
                <@com.picklist name.NameType/>
            </#local>
            <#local countries>
                <@com.picklistMultiple name.Country/>
            </#local>
            <#local identifier>
                <@com.text name.Name/> <#--Name is always empty in the table of  other identifiers for ICG component in MIM -->
            </#local>
            <#local remarks>
                <@printMultilingualField name.Remarks!{} true false/>
            </#local>
            <#local result= [{"type":type, "identifier": identifier?trim, "countries":countries, "remarks": remarks?trim}] + result/>
		</#list>
	</#if>
	<#return result?reverse/>
</#function>

<#function getOtherMixtureName mixture otherName>
	<#local result = ""/>
	<#if (mixture.OtherNames?has_content)>
		<#list mixture.OtherNames as name>
			<#if name.NameType?has_content && com.picklistValueMatchesPhrases(name.NameType, [".*" + otherName + ".*"])>
				<#if name.Name?has_content>
					<#--  We only return the first SF name found in the table  -->
					<#local result= name.Name/>
				</#if>
			</#if>
		</#list>
	</#if>
	<#return result/>
</#function>

<#macro displayUseTypes useTypes>
	<#if useTypes?seq_contains("consumer")>
		<figure class="col-md-auto">
			<i class="fa fa-home fa-3x" aria-hidden="true"></i>
			<figcaption>Consumer</figcaption>
		</figure>
	</#if>
	<#if useTypes?seq_contains("professional")>
		<figure class="col-md-auto">
			<i class="fa fa-hard-hat fa-3x" aria-hidden="true"></i>
			<figcaption>Professional</figcaption>
		</figure>
	</#if>
	<#if useTypes?seq_contains("industrial")>
		<figure class="col-md-auto">
			<i class="fa fa-industry fa-3x" aria-hidden="true"></i>
			<figcaption>Industrial</figcaption>
		</figure>
	</#if>
</#macro>

<#function getUseTypes mixtureProductInformation>
  <#local uses = [] />
  <#if mixtureProductInformation?has_content>
	<#list mixtureProductInformation as productInformation>
		<#if productInformation.AdditionalInformation.ProductUseCategory.UseType?has_content>
			<#list productInformation.AdditionalInformation.ProductUseCategory.UseType as useType>
				<#local use><@com.picklist useType/></#local>
				<#local uses = [use] + uses/>
			</#list>
    	</#if>
	</#list>
  </#if>
  <#return uses/>
</#function>

<#function getMainUses mixtureProductInformation>
  <#local uses = [] />
  <#if mixtureProductInformation?has_content>
	<#list mixtureProductInformation as productInformation>
		<#if productInformation.AdditionalInformation.ProductUseCategory.MainIntendedUse?has_content>
			<#local use><@picklistWoutAdditionalText productInformation.AdditionalInformation.ProductUseCategory.MainIntendedUse /></#local>
			<#local uses = [use] + uses/>
    	</#if>
	</#list>
  </#if>
  <#return uses/>
</#function>

<#function getSecondaryUses mixtureProductInformation>
  <#local uses = [] />
  <#if mixtureProductInformation?has_content>
	<#list mixtureProductInformation as productInformation>
		<#if productInformation.AdditionalInformation.ProductUseCategory.SecondaryUses?has_content>
			<#list productInformation.AdditionalInformation.ProductUseCategory.SecondaryUses as secondaryUse>
				<#local use><@com.picklist secondaryUse/></#local>
				<#local uses = [use] + uses/>
			</#list>
    	</#if>
	</#list>
  </#if>
  <#return uses/>
</#function>

<#--  Returns a string with the container and the sizes it comes from  -->
<#function createPackagingLabel document>
	<#local productPackaging = iuclid.getDocumentForKey(document)/>
	<#if productPackaging?has_content>
		<#local typeOfPackaging>
			<#if productPackaging.Packaging.TypeOfPackaging?has_content>
				<@com.picklist productPackaging.Packaging.TypeOfPackaging/>
			<#else>
				[Container type not reported]
			</#if>
		</#local>
		<#local packageSizes = []>
		<#list productPackaging.Packaging.SizeOfPackagingInContactWithTheProductContainerSize as size>
			<#local s><@com.range size.SizeOfPackaging/></#local>
			<#local packageSizes = [s] + packageSizes />
		</#list>
		<#if packageSizes?has_content>
			<#local sizes = "(" + packageSizes?reverse?join(', ') + ")"/>
		</#if>
		<#return typeOfPackaging + " " + sizes!/>
	</#if>
	<#return ""/>
</#function>

<#-- Returns a sequence of strings-->
<#function getTypeAndSizeOfPackages mixtureProductInformation>
	<#local result = []/>
	<#if mixtureProductInformation?has_content>
		<#list mixtureProductInformation as productInformation>
			<#if productInformation.AdditionalInformation.Packaging.ProductNotPackaged?has_content
			&& productInformation.AdditionalInformation.Packaging.ProductNotPackaged>
				<#local result = ["Not packaged"] + result />
			</#if>
			<#if productInformation.AdditionalInformation.Packaging.LinkToPackaging?has_content>
				<#list productInformation.AdditionalInformation.Packaging.LinkToPackaging as packaging>
					<#local result = [createPackagingLabel(packaging)] + result />
				</#list>
			</#if>
		</#list>
	</#if>
	<#return result?reverse>
</#function>

<#macro printDisableReason reason>
    <#switch reason>
        <#case "DISABLE_BY_MISTAKE">
            Submission made unintentionally
            <#break>
        <#case "DISABLE_TEST_DATA">
            Submitted dossier contains test data
            <#break>
        <#case "DISABLE_WRONG_COMPANY">
            Submission made by a wrong company
            <#break>
        <#case "DISABLE_WRONG_INFO">
            Submitted dossier contains wrong information
            <#break>
    </#switch>
</#macro>

<#macro printDisabledCard>
    <#if !(isDisabledSubmission)>
        <#return/>
    </#if>
    <div class="row">
        <div class="col-12">
            <div class="card">
                <div class="card-header py-0 bg-danger">
                    <h5 class="mb-0">
                        <button class="btn text-light">
                        Disabled submission
                        <i class="fas fa-ban"></i>
                        </button>
                    </h5>
                </div>

                <div>
                    <div class="card-body">
                        <dl class="row">
                            <dt class="col-sm-2">Disable date</dt>
                            <dd class="col-sm-2">
                               ${submissionContext.disableDate}
                            </dd>
                            <dt class="col-sm-2">Disable reason</dt>
                            <dd class="col-sm-5">
                                <@printDisableReason submissionContext.disableReason/>
                            </dd>
                        </dl>
                    </div>
                </div>
            </div> <!-- END disabling details-->
        </div>
    </div>
</#macro>

<#function getAttachmentUri attMetadata>
	<#local base = attachmentsPath/>
	<#local filename = attMetadata.md5 />
	<#local extension = "">
	<#if (attMetadata.filename?keep_after_last(".") != "")>
		<#local extension = attMetadata.filename?keep_after_last(".")/>
	</#if>
	<#local extension = extension?replace("[^a-zA-Z0-9]","", "r")/>
	<#return base + filename + "." + extension>
</#function>

<#macro listAttachments atts>
	<#local imageAttachments = []/>
	<#local otherAttachments = []/>
	<#local allAttachmentsFilenames = []/>
	<#list atts as att>
		<#if (att.AttachedDocument?has_content)>
			<#local metadata = iuclid.getMetadataForAttachment(att.AttachedDocument) />
             <#-- Avoid duplicates -->
			<#if !(allAttachmentsFilenames?seq_contains(metadata.filename))>
                <#if metadata.isImage>
					<#local imageAttachments = [metadata] + imageAttachments/>
				<#else>
					<#local otherAttachments = [metadata] + otherAttachments/>
				</#if>
			</#if>
            <#local allAttachmentsFilenames = [metadata.filename] + allAttachmentsFilenames/>
		</#if>
    <#else>
        <p>NA</p>
	</#list>
	<#--  We list first the  images  -->
	<#list imageAttachments?reverse as attMetadata>
		<#if (mode == "PUBLISHING")>
			<#--  <a href="${getAttachmentUri(attMetadata)}" target="_blank">
				<figure>
					<img class="attachment-img" src="${getAttachmentUri(attMetadata)}" alt="${(attMetadata.filename)}">
					<figcaption>${(attMetadata.filename)} <i class="fas fa-download"></i></figcaption>
				</figure>
			</a>  -->
			<#-- Workaround while https://dev.azure.com/echa-ecm/ECOMOD/_workitems/edit/626669 is fixed -->
			<p><a href="${getAttachmentUri(attMetadata)}" target="_blank">${(attMetadata.filename)} <i class="fas fa-download"></i></a></p>
		<#else>
			<p>${(attMetadata.filename)}</p>
		</#if>
	</#list>

	<#list otherAttachments?reverse as attMetadata>
		<#if (mode == "PUBLISHING")>
			<p><a href="${getAttachmentUri(attMetadata)}" target="_blank">${(attMetadata.filename)} <i class="fas fa-download"></i></a></p>
		<#else>
			<p>${(attMetadata.filename)}</p>
		</#if>
	</#list>
</#macro>

<#function getPackagingAttachments mixtureProductInformation>
	<#local result = []/>
	<#if mixtureProductInformation?has_content>
		<#list mixtureProductInformation as productInformation>
			<#if productInformation.AdditionalInformation.Packaging.LinkToPackaging?has_content>
				<#list productInformation.AdditionalInformation.Packaging.LinkToPackaging as packagingDoc>
					<#local productPackaging = iuclid.getDocumentForKey(packagingDoc)/>
					<#local attachmentInfo = []/>
					<#if productPackaging?has_content>
						<#if productPackaging.Packaging.PackagingRelatedAttachments?has_content>
							<#list productPackaging.Packaging.PackagingRelatedAttachments as attachment>
								<#local result = [attachment] + result />
							</#list>
						</#if>
					</#if>
				</#list>
			</#if>
		</#list>
	</#if>
	<#return result?reverse>
</#function>

<#function getRegulatoryIdentifiersFromRecord record identifierName>
	<#local identifiers = [] />
    <#if record.RegulatoryProgrammeIdentifiers.RegulatoryProgrammeIdentifiers?has_content>
        <#list record.RegulatoryProgrammeIdentifiers.RegulatoryProgrammeIdentifiers as identifiersProgramme>
            <#if identifiersProgramme.RegulatoryProgramme?has_content && com.picklistValueMatchesPhrases(identifiersProgramme.RegulatoryProgramme, [identifierName])>
                <#if identifiersProgramme.Id?has_content>
                    <#local identifiers = [identifiersProgramme.Id] + identifiers />
                </#if>
            </#if>
        </#list>
    </#if>
	<#return identifiers?reverse>
</#function>

<!-- Returns a sequence of strings-->
<#function getRegulatoryIdentifiers mixture identifierName>
	<#local identifierRecords = iuclid.getSectionDocumentsForParentKey(mixture.documentKey, "FLEXIBLE_RECORD", "Identifiers") />
	<#local identifiers = [] />
	<#list identifierRecords as record>
        <#local identifiers = getRegulatoryIdentifiersFromRecord(record, identifierName) + identifiers />
	</#list>
	<#return identifiers>
</#function>

<#function getOtherITIdentifiers mixture>
	<#local identifierRecords = iuclid.getSectionDocumentsForParentKey(mixture.documentKey, "FLEXIBLE_RECORD", "Identifiers") />
	<#local identifiers = [] />
	<#list identifierRecords as record>
		<#if record.ExternalSystemIdentifiers.ExternalSystemIdentifiers?has_content>
			<#list record.ExternalSystemIdentifiers.ExternalSystemIdentifiers as identifier>
				<#local iTSystem>
					<#if (identifier.ExternalSystemDesignator?has_content)>
						<@com.text identifier.ExternalSystemDesignator/>
					<#else>
						NA
					</#if>
				</#local>
				<#local id>
					<#if (identifier.Id?has_content)>
						<@com.text identifier.Id/>
					<#else>
						NA
					</#if>
				</#local>
				<#local identifiers = [{"iTSystem": iTSystem, "id": id}] + identifiers/>
			</#list>
		</#if>
	</#list>
	<#return identifiers?reverse>
</#function>

<#function getColourAndPhysicalForProduct colourAndPhysicalInfo>
	<#local colourAndPhysical = iuclid.getDocumentForKey(colourAndPhysicalInfo)/>
	<#local colours = []/>
	<#local form = ""/>
	<#local physicalState = ""/>
	<#local colourIntensity = ""/>

	<#if colourAndPhysical?has_content>
		<#if colourAndPhysical.KeyValueForChemicalSafetyAssessment.Colour?has_content>
			<#list colourAndPhysical.KeyValueForChemicalSafetyAssessment.Colour as colour>
				<#local c><@com.picklist colour /></#local>
				<#local colours = [c] + colours />

			</#list>
		</#if>
		<#if colourAndPhysical.KeyValueForChemicalSafetyAssessment.Form?has_content>
			<#local form><@com.picklist colourAndPhysical.KeyValueForChemicalSafetyAssessment.Form /></#local>
		</#if>
		<#if colourAndPhysical.KeyValueForChemicalSafetyAssessment.PhysicalState?has_content>
			<#local physicalState><@com.picklist colourAndPhysical.KeyValueForChemicalSafetyAssessment.PhysicalState /></#local>
		</#if>
		<#if colourAndPhysical.KeyValueForChemicalSafetyAssessment.ColourIntensity?has_content>
			<#local colourIntensity><@com.picklist colourAndPhysical.KeyValueForChemicalSafetyAssessment.ColourIntensity /></#local>
		</#if>
	</#if>
	<#return { "colours": colours?reverse, "form": form, "physicalState": physicalState, "colourIntensity": colourIntensity}/>
</#function>

<#macro listStrings l newLine=false html=false>
	<#if !(l?has_content) || (l?first == "" && l?size == 1)>
        <#if (html)>
		    <p>NA</p>
        <#else>
            NA
        </#if>
		<#return/>
	</#if>
    <#if (html)>
        <#if (newLine)>
            <#list l as e>
                <p>${e}</p>
            </#list>
        <#else>
            <#list l as e>
                ${e}
            </#list>
	    </#if>
    <#elseif (newLine)>
		<#list l as e>
			<p><@com.text e/></p>
		</#list>
    <#else>
		<@com.text l?join(', ', 'NA') />
	</#if>
</#macro>

<#macro printIdentifiersForComponent l newLine=false html=false>
<#-- Macro used in the component summary of mixture composition table
     We dont want synomyns in this printout of names -->
	<#if !(l?has_content)>
		NA
		<#return/>
	</#if>

	<#list l?keys as key>
		<#if (l[key]?has_content) && (key != "Synonyms")>
			<#local value>
				<#if (l[key]?is_enumerable)>
					<@listStrings l[key] newLine html/>
				<#else>
					<@com.text l[key]/>
				</#if>
			</#local>
			<strong>${key}</strong>: ${value}
		</#if>
	</#list>
</#macro>

<#macro printColours colours>
	<#list colours as colour>
		<#if (colour == "colourless")>
			${colour}
		<#else>
			${colour} <span class="color-dot" style="background-color: ${colour};"></span>
		</#if>
    <#else>
        NA
	</#list>
</#macro>

<#macro printStot type item>
		<h6>STOT-${type}</h6>
        <div class="pl-4">
            <dl class="row">
                <dt class="col-sm-3">Hazard category</dt>
                <dd class="col-sm-9">
                    <p>
                        <@com.picklist item.Toxicity.HazardCategory/>
                    </p>
                </dd>
                <dt class="col-sm-3">Hazard statement</dt>
                <dd class="col-sm-9">
                    <#local statement><@com.picklist item.Toxicity.HazardStatement/></#local>
                    <p>
                        ${cleanStatement(statement)}
                    </p>
                </dd>
                <dt class="col-sm-3">System</dt>
                <dd class="col-sm-9">
                    <@printMultilingualPicklist item.Toxicity.System false true/>
                </dd>
                <dt class="col-sm-3">Affected organs</dt>
                <dd class="col-sm-9">
                    <@printMultilingualPicklistMultiple item.Toxicity.AffectedOrgans "" true true/>
                </dd>
                <dt class="col-sm-3">Route of exposure</dt>
                <dd class="col-sm-9">
                    <@printMultilingualPicklistMultiple item.Toxicity.RouteExposure "" true true />
                </dd>
            </dl>
        </div>
</#macro>

<#function removeDuplicates seq>
	<#if !(seq?has_content)>
		<#return seq/>
	</#if>
	<#local result = []/>
	<#list seq as item>
		<#if !(result?seq_contains(item))>
			<#local result = [item] + result />
		</#if>
	</#list>
	<#return result?reverse/>
</#function>

<#function createHash object>
    <#local hash = ""/>
    <#local keys = object?keys?sort>
    <#list keys as key>
        <#local hash = hash + key + "." + object[key] + "'"/>
    </#list>
    <#return hash>
</#function>

<#function removeDuplicatedObjectsInList seq>
	<#if !(seq?has_content)>
		<#return seq/>
	</#if>
    <#local mergedValuesList = []/>
	<#local result = []/>
	<#list seq as item>
        <#local mergedValues = createHash(item) />
		<#if !(mergedValuesList?seq_contains(mergedValues))>
			<#local result = [item] + result />
            <#local mergedValuesList = [mergedValues] + mergedValuesList />
		</#if>
	</#list>
	<#return result?reverse/>
</#function>

<#function isObjectInList obj seq>
	<#if !(seq?has_content)>
		<#return false/>
	</#if>
    <#local objHash = createHash(obj)/>
	<#list seq as item>
        <#if createHash(item) == objHash>
            <#return true/>
        </#if>
	</#list>
	<#return false/>
</#function>

<#function mergeObjectValuesFromObjectList objectList>
    <#if !(objectList?has_content)>
		<#return objectList/>
	</#if>
    <#local result = {}/>
    <#list objectList[0]?keys as key>
        <#local result = result + {key: []}/>
    </#list>
    <#list objectList as item>
        <#list result?keys as key>
            <#if (item[key]?has_content)>
                <#--  Complex way of building a complex object in Freemarker  -->
                <#local result = result + {key: [item[key]] + result[key]} />
            </#if>
        </#list>
    </#list>
    <#return result />
</#function>

<#macro printCountriesList countriesList>
    <#assign markets>
        <@com.picklistMultiple countriesList/>
    </#assign>
    <@pcn.listStrings markets?split(';') />
</#macro>

<#function removeHazardStatements l>
	<#local result = []>
	<#list l as entry>
		<#local result = [entry?keep_before(':')] + result />
	</#list>
	<#return result?reverse>
</#function>

<#function cleanStatement statement>
	<#local start = statement?index_of("&lt;")/>
	<#local end = statement?index_of("&gt;") + 4/>
	<#if (start > -1 && end > -1 )>
		<#return statement?substring(0, start) + cleanStatement(statement?substring(end)) />
	<#else>
		<#return statement/>
	</#if>
</#function>

<#function getColourAndPhysical mixtureProductInformation>
	<#local colour = []/>
	<#local form = []/>
	<#local physicalState = []/>
	<#local colourIntensity = []/>

	<#list mixtureProductInformation as productInformation>
		<#if productInformation.AdditionalInformation.ColourAndPhysicalState.LinkToColourAndPhysicalState?has_content>
			<#list productInformation.AdditionalInformation.ColourAndPhysicalState.LinkToColourAndPhysicalState as colourAndPhysicalState>
				<#local state = getColourAndPhysicalForProduct(colourAndPhysicalState)/>
				<#local colour = state.colours + colour />
				<#local form = [state.form] + form />
				<#local physicalState = [state.physicalState] + physicalState />
				<#local colourIntensity = [state.colourIntensity] + colourIntensity />
			</#list>
		</#if>
	</#list>
    <#-- Given that we are merging several docs, we sort the information alphabetically -->
	<#return { "colour": colour?sort, "form": form?sort, "physicalState": physicalState?sort, "colourIntensity": colourIntensity?sort!""}/>
</#function>

<#macro picklistWoutAdditionalText picklistValue locale="en" printOtherPhrase=false>
<#compress>
	<#escape x as x?html>
		<#local localizedPhrase = iuclid.localizedPhraseDefinitionFor(picklistValue.code, locale) />
		<#if localizedPhrase?has_content>
			<#if !localizedPhrase.open || !(localizedPhrase.text?matches("other:")) || printOtherPhrase>
				${localizedPhrase.text} <#t>
			</#if>
			<#if localizedPhrase.open && picklistValue.otherText?has_content>
				${picklistValue.otherText}<#t>
			</#if>
		</#if>
		<#if picklistValue.remarks?has_content>
			- ${picklistValue.remarks}
		</#if>
		<#lt>
	</#escape>
</#compress>
</#macro>

<#macro printSubmissionDetails dossierHeader>
	<#if !headerHasContents>
		<p>Not a dossier</p>
		<#return/>
	</#if>
	<#local limited = dossierHeader.SpecificSubmissions.SubmissionType.LimitedSubmission />
	<#local group = dossierHeader.SpecificSubmissions.SubmissionType.GroupSubmission />
	<#local voluntarySubmission = dossierHeader.SpecificSubmissions.SubmissionType.VoluntarySubmission />
	<#if limited>
		<p>Limited submission</p>
	</#if>
	<#if group>
		<p>Group submission</p>
	</#if>
	<#if voluntarySubmission>
		<p>Voluntary submission</p>
	</#if>
	<#if (!limited && !group && !voluntarySubmission)>
		<p>Standard</p>
	</#if>
</#macro>

<#macro printNotificationDetails dossierHeader tab>
	<#if !headerHasContents>
        <dd class="col-sm-12">Not a dossier</dd>
		<#return />
	</#if>
	<#local initialNotification = dossierHeader.NotificationType.InitialNotif />
	<#local changeInComposition = dossierHeader.NotificationType.NewNotifChangeCompo />
	<#local update = dossierHeader.NotificationType.SubmissionIsAnUpdate />
	<dt class="col-sm-3">Notification type</dt>
	<dd class="col-sm-9">
		<#if initialNotification>
			<p>Initial notification</p>
		<#elseif changeInComposition>
			<p>New notification after a significant change of composition</p>
		<#elseif update>
			<p>Updated submission</p>
		<#else>
			<p>NA</p>
		</#if>
	</dd>

	<#local reasonForUpdating = dossierHeader.NotificationType.ReasonForUpdating.AfterSpontaneousUpdate />
	<#if reasonForUpdating?has_content>
		<#list reasonForUpdating as updateReasons>
			<#if updateReasons.Justification?has_content>
				<#local justification><@com.picklist updateReasons.Justification /></#local>
				<#local justificationId = justification?replace(" ","_")/>
				<dt class="col-sm-3">Justification for update</dt>
				<dd class="col-sm-9">${justification}
					<#if ((updateReasons.Remarks.localizedTexts)?has_content)>
					<div>
						<a class="font-weight-normal collapsed" data-toggle="collapse" data-target="#${justificationId}-${tab}">
							<i class="toggle fa"></i>
							Justification remarks
						</a>
					</div>
					<div id="${justificationId}-${tab}" class="collapse d-print-block" >
						<@printMultilingualField updateReasons.Remarks false false/>
					</div>
					</#if>
				</dd>
				<#--  <div><b>Justification remarks</b></div>
				<@printMultilingualField updateReasons.Remarks false true/></dd>  -->
			</#if>
		</#list>
	</#if>
	<#if dossierHeader.NotificationType.ReasonForUpdating.OtherUpdateReason?has_content>
		<dt class="col-sm-3">Other update reason</dt>
		<dd class="col-sm-9">
			<@printMultilingualField dossierHeader.NotificationType.ReasonForUpdating.OtherUpdateReason/>
		</dd>
	</#if>
</#macro>

<#macro printLabellingHeader record>
    <#if !(record?has_content)>
        <#return/>
    </#if>
    <#local isClassified = !(record.GeneralInformation.NotClassified)!false />
    <#if (isClassified)>
        <h4>
	        <#if (record.Labelling.SignalWord)?has_content>
		        <#local signalWord><@com.picklist record.Labelling.SignalWord /></#local>
                <span class="badge badge-pill badge-${signalWord?lower_case}">${signalWord}</span>
        	</#if>

        	<#if (record.Labelling.HazardPictogramBlock.HazardPictogram)?has_content>
                <@HazardPictogramList record.Labelling.HazardPictogramBlock.HazardPictogram/>
        	</#if>
        </h4>
    </#if>
</#macro>

<#macro printLabelling record>
    <#local isClassified = !record.GeneralInformation.NotClassified />
	<#if !(record.Labelling.SignalWord?has_content || record.Labelling.HazardPictogramBlock.HazardPictogram?has_content ||  record.Labelling.HazardStatementsBlock.HazardStatements?has_content || record.Labelling.PrecautionaryStatementsBlock.PrecautionaryStatements?has_content)>
		<dt class="font-weight-normal col-12">NA</dt><dd></dd>
		<#return>
	</#if>
	<#if record.Labelling.SignalWord?has_content && isClassified>
		<#local signalWord><@com.picklist record.Labelling.SignalWord /></#local>
		<dt class="col-sm-3">Signal word</dt>
		<dd class="col-sm-9">
			<span class="badge badge-pill badge-${signalWord?lower_case}">${signalWord}</span>
        </dd>
	</#if>

	<#if record.Labelling.HazardPictogramBlock.HazardPictogram?has_content && isClassified>
		<dt class="col-sm-3">Pictograms</dt>
		<dd class="text-center row col-sm-9">
			<@HazardPictogramList record.Labelling.HazardPictogramBlock.HazardPictogram/>
		</dd>
	</#if>
	<#if record.Labelling.HazardStatementsBlock.HazardStatements?has_content>
		<dt class="col-sm-3">Hazard statements</dt>
        <dd class="col-sm-9">
			<@HazardStatementList record.Labelling.HazardStatementsBlock.HazardStatements/>
		</dd>
	</#if>
	<#if record.Labelling.PrecautionaryStatementsBlock.PrecautionaryStatements?has_content>
		<dt class="col-sm-3">Precautionary statements</dt>
        <dd class="col-sm-9">
			<@PrecautionaryStatementsList record.Labelling.PrecautionaryStatementsBlock.PrecautionaryStatements/>
		</dd>
	</#if>
</#macro>

<#macro printAdditionalLabelling record>
	<#if !(record.Labelling.LabelingRequirementsBlock?has_content)>
		<dt class="font-weight-normal col-12">NA</dt><dd></dd>
		<#return>
	</#if>
	<#if record.Labelling.LabelingRequirementsBlock.LabelingRequirements?has_content>
		<dt class="col-sm-3">Additional non-GHS hazard statements</dt>
        <dd class="col-sm-9">
			<@LabellingRequirementsList record.Labelling.LabelingRequirementsBlock.LabelingRequirements/>
		</dd>
	</#if>
    <#if record.Labelling.LabelingRequirementsBlock.AdditionalLabelling?has_content>
		<dt class="col-sm-3">Additional labelling requirements</dt>
        <dd class="col-sm-9">
            <dl>
                <#list record.Labelling.LabelingRequirementsBlock.AdditionalLabelling as blockItem>
                    <dt></dt><dd><@printMultilingualField blockItem.Labelling false false/></dd>
                </#list>
            </dl>
        </dd>
	</#if>
</#macro>

<#macro HazardPictogramList HazardPictogramRepeatableBlock>
	<#if HazardPictogramRepeatableBlock?has_content>
		<#list HazardPictogramRepeatableBlock as blockItem>
			<#if blockItem.Code.code?has_content>
				<#local code><@com.picklist blockItem.Code/></#local>
				<div class="align-middle pictogram ${code?keep_before(':')?lower_case}" alt="${code}"></div>
			</#if>
		</#list>
	</#if>
</#macro>

<#macro HazardStatementList HazardStatementRepeatableBlock>
	<#if HazardStatementRepeatableBlock?has_content>
		<dl>
			<#list HazardStatementRepeatableBlock as blockItem>
				<#local statement><@com.picklist blockItem.HazardStatement/></#local>
				<dt class="font-weight-normal">
					${cleanStatement(statement)}
				</dt>
				<dd class="font-weight-light">
					<@printMultilingualField blockItem.AdditionalText false false/>
				</dd>
			</#list>
		</dl>
  	</#if>
</#macro>


<#macro PrecautionaryStatementsList PrecautionaryStatementRepeatableBlock>
	<#if PrecautionaryStatementRepeatableBlock?has_content>
		<dl>
			<#list PrecautionaryStatementRepeatableBlock as blockItem>
				<dt class="font-weight-normal">
					<@com.picklist blockItem.PrecautionaryStatement/>
				</dt>
				<dd class="font-weight-light">
					<@printMultilingualField blockItem.AdditionalText false false/>
				</dd>
			</#list>
		</dl>
  	</#if>
</#macro>

<#macro LabellingRequirementsList LabellingRequirementsRepeatableBlock>
	<#if LabellingRequirementsRepeatableBlock?has_content>
		<dl>
			<#list LabellingRequirementsRepeatableBlock as blockItem>
				<dt class="font-weight-normal">
					<@com.picklist blockItem.SupplHazardStatement/>
				</dt>
				<dd class="font-weight-light">
					<@printMultilingualField blockItem.AdditionalText false false/>
				</dd>
			</#list>
		</dl>
  	</#if>
</#macro>

<#function getSdsList component>
	<#local sdsAttachmentList = []/>
	<#local sdsSection11List = []/>
    <#local hasMixturesLinked = false/>
	<#local recordList = iuclid.getSectionDocumentsForParentKey(component.documentKey, "FLEXIBLE_RECORD", "SDSInfoMixtures") />
	<#if recordList?has_content>
		<#list recordList as record>
            <#local mixtures = []/>
            <#if (record.InformationOnMixtures.RelatedComposition?has_content)>
                <#local mixtures = buildMixtureReferences(record.InformationOnMixtures.RelatedComposition)/>
                <#local hasMixturesLinked = true/>
            </#if>
			<#local toxInfo = record.InformationOnMixtures.TradeNamesAndSafetyDataSheetsOfTheMixture.SDSsection11/>
			<#if toxInfo.localizedTexts?has_content>
				<#list toxInfo.localizedTexts as toxicologicalInformation>
					<#if (toxicologicalInformation.value?has_content)>
						<#local text = toxicologicalInformation.value/>
					</#if>
					<#local language><@com.text toxicologicalInformation.language/></#local>
                    <#local language = language?trim/>
					<#local languageName = getLanguageName(language)/>
					<#local sdsSection11List = [{"text": text!"", "language": language, "languageName": languageName, "mixtures": mixtures}] + sdsSection11List />
				</#list>
			<#elseif !(toxInfo.localizedTexts?has_content)>
				<#local text = toxInfo?html/>
				<#local sdsSection11List = [{"text": text, "language": "default", "languageName": "default", "mixtures": mixtures}] + sdsSection11List />
			</#if>
			<#local safetyDataSheetsOnMixtures = record.InformationOnMixtures.TradeNamesAndSafetyDataSheetsOfTheMixture.TradeNamesAndSafetyDataSheets />
			<#list safetyDataSheetsOnMixtures as sds>
				<#if sds?has_content>
					<#local attachment = sds.Attachment />
					<#if attachment?has_content>
						<#local attachmentData = iuclid.getMetadataForAttachment(attachment) />
						<#if attachmentData?has_content>
							<#local language><@com.picklist sds.Language/></#local>
							<#local country><@com.picklist sds.Country/></#local>
                            <#if (language == "")>
                                <#local language>NA</#local>
                            </#if>
							<#local sdsAttachmentList = [{"countryCode": getCountryCode(country),"filename": attachmentData.filename?html, "language": language?keep_before(" "), "uri": getAttachmentUri(attachmentData), "mixtures": mixtures}] + sdsAttachmentList />
						</#if>
					</#if>
				</#if>
			</#list>
		</#list>
	</#if>
	<#return {"sdsSection11List": sdsSection11List?sort_by("languageName"), "sdsAttachmentList": sdsAttachmentList?reverse, "hasMixturesLinked": hasMixturesLinked}/>
</#function>

<#function getHealthHazards record>
	<#return {
	"Acute toxicity - oral:" : record.Classification.HealthHazards.AcuteToxicityOral,
	"Acute toxicity - dermal:" : record.Classification.HealthHazards.AcuteToxicityDermal,
	"Acute toxicity - inhalation:" : record.Classification.HealthHazards.AcuteToxicityInhalation,
	"Skin corrosion / irritation:" : record.Classification.HealthHazards.Irritation,
	"Serious damage / eye irritation:" : record.Classification.HealthHazards.EyeIrritation,
	"Respiratory sensitisation:" : record.Classification.HealthHazards.RespiratorySensitisation,
	"Skin sensitisation:" : record.Classification.HealthHazards.SkinSensitisation,
	"Aspiration hazard:" : record.Classification.HealthHazards.AspirationHazard
	}/>
</#function>

<#function getPhysicalHazards record>
	<#return {
	"Explosives:" : record.Classification.PhysicalHazards.Explosives,
	"Desensitised explosives:" : record.Classification.PhysicalHazards.DesensitizedExplosives,
	"Flammable gases and chemically unstable gases:" : record.Classification.PhysicalHazards.FlammableGases,
	"Aerosols:" : record.Classification.PhysicalHazards.FlammableAerosols,
	"Chemicals under Pressure:" : record.Classification.PhysicalHazards.ChemicalsUnderPressure,
	"Oxidising gases:" : record.Classification.PhysicalHazards.OxidisingGases,
	"Gases under pressure:" : record.Classification.PhysicalHazards.GasesPres,
	"Flammable liquids:" : record.Classification.PhysicalHazards.FlammableLiquids,
	"Flammable solids:" : record.Classification.PhysicalHazards.FlammableSolids,
	"Self-reactive substances and mixtures:" : record.Classification.PhysicalHazards.SelfReactiveSubstMixt,
	"Pyrophoric liquids:" : record.Classification.PhysicalHazards.PyrophoricLiquids,
	"Pyrophoric solids:" : record.Classification.PhysicalHazards.PyrophoricSolids,
	"Self-heating substances and mixtures:" : record.Classification.PhysicalHazards.SelfHeatSubstMixt,
	"Substances and mixtures which in contact with water emit flammable gases:" : record.Classification.PhysicalHazards.SubstMixtWater,
	"Oxidising liquids:" : record.Classification.PhysicalHazards.OxidisingLiquids,
	"Oxidising solids:" : record.Classification.PhysicalHazards.OxidisingSolids,
	"Organic peroxides:" : record.Classification.PhysicalHazards.OrganicPeroxides,
	"Corrosive to metals:" : record.Classification.PhysicalHazards.CorMetals
	}/>
</#function>

<#function hazardClassificationToCompactString classification>
	<#local c><@com.picklist classification.HazardCategory/></#local>
	<#local s><@com.picklist classification.HazardStatement/></#local>
	<#if (c?has_content && s?has_content)>
		<#local string = c + "-" + s/>
	<#elseif (c?has_content)>
		<#local string = c/>
	<#elseif (s?has_content)>
		<#local string = s/>
	</#if>
	<#return string />
</#function>
<!-- hazardObjectToList removes empty entries -->
<#function hazardsObjectToList hazardsObject keepEmpty=false>
	<#local result = [] />
	<#local hazardTypes = hazardsObject?keys />

	<#list hazardTypes as type>
		<#if keepEmpty>
			<#local entry = {"type": type, "content": hazardsObject[type]} />
			<#local result = [entry] + result/>
		<#elseif isNotEmptyClassificationBlock(hazardsObject[type])>
			<#local entry = {"type": type, "content": hazardsObject[type]} />
			<#local result = [entry] + result/>
		</#if>
		<#--  <#elseif includeNAHazards>
			<#local entry = {"type": type, "content": "} />
			<#local result = [entry] + result/>  -->
		<#--  </#if>  -->
	</#list>
	<#return result?reverse/>
</#function>

<#function getCompositionsIndex mixtureCompositionList>
    <#local index = {}/>
    <#list mixtureCompositionList as composition>
        <#local index = index + {composition.documentKey: composition?counter}/>
    </#list>
    <#return index>
</#function>

<#macro printCompositionCard composition commonComponents=[] commonCard=false compositionId=0>
    <div class="row card-row-${submissionStatus}">
        <div class="col-12">
            <div class="card">
                <div class="card-header py-0" id="mixtureCompCard-${compositionId}">

                    <#if (commonCard)>
                        <h5 class="mb-0">
                            <button class="btn btn-link" data-toggle="collapse"
                                data-target="#mixtureCompositionBody-${compositionId}">
                                <i class="toggle fa"></i>
                                Common components of mixture compositions
                                <i class="fas fa-layer-group" title="Group submission"></i>
                            </button>
                        </h5>
                    <#elseif compositionId == 0>
                        <h5 class="btn mb-0">
                            Mixture composition
                        </h5>
                    <#else>
                        <h5 class="btn mb-0">
                            <a id="mixtureComp-${compositionId}"></a>Specific components of mixture #${compositionId}
                        </#if>
                    </h5>
                </div>
                <#if (commonCard)>
                     <div class="card-body collapse show" id="mixtureCompositionBody-${compositionId}">
                <#else>
                     <div class="card-body">
                </#if>

                    <#if (!commonCard && (commonComponents?size == composition.Components.Components?size) && (composition.Components.Components?size > 0))>
                        <p>All components of this mixture composition are common.</p>
                    <#else>
                        <table class="table">
                            <thead>
                                <tr>
                                    <th class="componentNameColumn">Name</th>
                                    <th>Function</th>
                                    <th>Concentration (w/w)</th>
                                    <th>Classification</th>
                                    <th>Labelling</th>
                                </tr>
                            </thead>
                            <tbody>
                                <#local componentId = 1>
                                <#local components = addSortValue(composition.Components.Components) />
                                <#list sortByConcentration(components) as component>
                                    <#local isCommon = isInCommonComponentList(component.sortedComponent commonComponents)/>
                                    <#if (commonCard && isCommon || (!commonCard && !isCommon))>
                                        <#local stringId>
                                            <#if (compositionId?is_number && compositionId != 0)>
                                                <#local componentId = 1>
                                                M${compositionId}-${componentId}
                                            <#else>
                                                ${componentId}
                                            </#if>
                                        </#local>
                                        <@printComponent component stringId?trim  0/>
                                    </#if>
                                    <#local componentId = componentId + 1>
                                </#list>
                            </tbody>
                        </table>
                    </#if>
                </div>
            </div>
        </div>
    </div>
</#macro>

<#function getCompactClassification record>
    <#local isClassified = !record.GeneralInformation.NotClassified />
    <#local classification = []/>
	<#if (!isClassified)>
    		<#local classification = ["Not classified"] + classification/>
	</#if>

    <#if (isClassified)>
        <#local healthHazardClassifications = hazardsObjectToList(getHealthHazards(record))/>
        <#local physicalHazardClassifications = hazardsObjectToList(getPhysicalHazards(record))/>


        <#list healthHazardClassifications as hazard>
            <#local classification = [hazardClassificationToCompactString(hazard.content)] + classification/>
        </#list>

        <#if (isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.ReproductiveToxicity.ReproductiveToxicity))>
            <#local classification = [hazardClassificationToCompactString(record.Classification.HealthHazards.ReproductiveToxicity.ReproductiveToxicity)] + classification/>
        </#if>

        <#if (record.Classification.HealthHazards.ReproductiveToxicity.Effects?has_content) &&
            (record.Classification.HealthHazards.ReproductiveToxicity.Effects.HazardCategory?has_content)>
            <#local classification = [hazardClassificationToCompactString(record.Classification.HealthHazards.ReproductiveToxicity.Effects)] + classification/>
        </#if>

        <#if (isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.GermCell.GermCell))>
            <#local classification = [hazardClassificationToCompactString(record.Classification.HealthHazards.GermCell.GermCell)] + classification/>
        </#if>

        <#if (isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.Carcinogenicity.Carcinogenicity))>
            <#local classification = [hazardClassificationToCompactString(record.Classification.HealthHazards.Carcinogenicity.Carcinogenicity)] + classification/>
        </#if>

        <#list record.Classification.HealthHazards.ToxicitySingle.Toxicity as item>
            <#if isClassificationSpecificTargetOrgan(item)>
                <#if item.Toxicity.HazardCategory?has_content>
                    <#local classification = [hazardClassificationToCompactString(item.Toxicity)] + classification/>
                </#if>
            </#if>
        </#list>

        <#list record.Classification.HealthHazards.ToxicityRepeated.Toxicity as item>
            <#if isClassificationSpecificTargetOrgan(item)>
                <#if item.Toxicity.HazardCategory?has_content>
                    <#local classification = [hazardClassificationToCompactString(item.Toxicity)] + classification/>
                </#if>
            </#if>
        </#list>

        <#list physicalHazardClassifications as hazard>
            <#local classification = [hazardClassificationToCompactString(hazard.content)] + classification/>
        </#list>

        <#if isClassificationEnvironmentalHazards(record)>
            <#local environmentalHazards = record.Classification.EnvironmentalHazards />
            <#if isNotEmptyClassificationBlock(environmentalHazards.AquaticEnvironment.AcuteShortTerm)>
                <#local classification = [hazardClassificationToCompactString(environmentalHazards.AquaticEnvironment.AcuteShortTerm)] + classification/>
            </#if>
            <#if isNotEmptyClassificationBlock(environmentalHazards.AquaticEnvironment.LongTerm)>
                <#local classification = [hazardClassificationToCompactString(environmentalHazards.AquaticEnvironment.LongTerm)] + classification/>
            </#if>
            <#if isNotEmptyClassificationBlock(environmentalHazards.OzoneLayer.HazardousOzone)>
                <#local classification = [hazardClassificationToCompactString(environmentalHazards.OzoneLayer.HazardousOzone)] + classification/>
            </#if>
        </#if>
        <#if isClassificationSpecificConcLimits(record)>
                <#list record.Classification.SpecificConcentrations.ConcentrationRange as item>
                    <#if isNotEmptyConcentrationBlock(item)>
                        <#local compactValue>
                            <span class="d-table"><@com.range item.ConcentrationRangeVal/>% <@com.picklistMultiple item.HazardCategories/></span>
                        </#local>
                        <#local classification = [compactValue] + classification/>
                    </#if>
                </#list>
        </#if>
    </#if>

    <#if (record.Classification.AdditionalHazard.Classes?has_content)>
        <#local additionalHazardValue>
            <@printMultilingualField record.Classification.AdditionalHazard.Classes/>
        </#local>
        <#local classification = [additionalHazardValue] + classification/>
    </#if>

    <#if (record.Classification.AdditionalHazard.Statements?has_content)>
        <#local additionalHazardValue>
            <@printMultilingualField record.Classification.AdditionalHazard.Statements/>
        </#local>

        <#local classification = [additionalHazardValue] + classification/>
    </#if>

	<#return classification?reverse/>
</#function>

<#macro ghsClassification record>
    <#local isClassified = !record.GeneralInformation.NotClassified />
	<#local healthHazardClassifications = hazardsObjectToList(getHealthHazards(record))/>
	<#if isClassificationHealtHazards(record, healthHazardClassifications)>
		<div class="healthClassification">
			<h5>Health hazards</h5>
            <#if (isClassified)>
                <dl class="row">
                    <#list healthHazardClassifications as hazard>
                            <dt class="col-sm-3">${hazard.type}</dt>
                            <dd class="col-sm-9">
                                <p><@com.picklist hazard.content.HazardCategory/>-<@com.picklist hazard.content.HazardStatement!/></p>
                            </dd>
                    </#list>
                </dl>
                <#list record.Classification.HealthHazards.ToxicitySingle.Toxicity as item>
                    <#if isClassificationSpecificTargetOrgan(item)>
                        <@printStot "SE" item/>
                    </#if>
                </#list>

                <#list record.Classification.HealthHazards.ToxicityRepeated.Toxicity as item>
                    <#if isClassificationSpecificTargetOrgan(item)>
                        <@printStot "RE" item/>
                    </#if>
                </#list>

                <#if isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.ReproductiveToxicity.ReproductiveToxicity) || isNotEmptyClassificationBlock(record.Classification.HealthHazards.ReproductiveToxicity.Effects)>
                    <#local reproductiveToxicity = record.Classification.HealthHazards.ReproductiveToxicity.ReproductiveToxicity/>
                    <h6>Reproductive toxicity</h6>
                    <div class="pl-4">
                        <dl class="row">
                            <dt class="col-sm-3">Hazard category</dt>
                            <dd class="col-sm-9">
                                <@com.picklist reproductiveToxicity.HazardCategory/>
                            </dd>
                            <#if (reproductiveToxicity.HazardStatement?has_content)>
                                <dt class="col-sm-3">Hazard statement</dt>
                                <dd class="col-sm-9">
                                    <@com.picklist reproductiveToxicity.HazardStatement/>
                                </dd>
                            </#if>
                            <dt class="col-sm-3">Specific effect</dt>
                            <dd class="col-sm-9">
                                <@printMultilingualField reproductiveToxicity.SpecificEffect/>
                                </dd>
                            <dt class="col-sm-3">Route of exposure</dt>
                            <dd class="col-sm-9">
                                <p><@printMultilingualPicklistMultiple reproductiveToxicity.RouteExposure ""/></p>
                            </dd>
                            <dt class="col-sm-3">Effects on lactation</dt>
                            <dd class="col-sm-9">
                                <#if (isNotEmptyClassificationBlock(record.Classification.HealthHazards.ReproductiveToxicity.Effects))>
                                    <p><@com.picklist record.Classification.HealthHazards.ReproductiveToxicity.Effects.HazardCategory/>-<@com.picklist record.Classification.HealthHazards.ReproductiveToxicity.Effects.HazardStatement/></p>
                                <#else>
                                    <p>NA</p>
                                </#if>
                            </dd>
                        </dl>
                    </div>
                </#if>

                <#if (isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.GermCell.GermCell))>
                    <h6>Germ cell mutagenicity</h6>
                    <div class="pl-4">
                        <dl class="row">
                            <#local germCell = record.Classification.HealthHazards.GermCell.GermCell/>
                            <dt class="col-sm-3">Hazard category</dt>
                            <dd class="col-sm-9">
                                <@com.picklist germCell.HazardCategory/>
                            </dd>
                            <dt class="col-sm-3">Hazard statement</dt>
                            <dd class="col-sm-9">
                                <@com.picklist germCell.HazardStatement/>
                            </dd>
                            <dt class="col-sm-3">Route of exposure</dt>
                            <dd class="col-sm-9">
                                <p><@printMultilingualPicklistMultiple germCell.RouteExposure ""/></p>
                            </dd>
                        </dl>
                    </div>
                </#if>

                <#if (isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.Carcinogenicity.Carcinogenicity))>
                    <#local carcinogenicity = record.Classification.HealthHazards.Carcinogenicity.Carcinogenicity/>
                    <h6>Carcinogenicity</h6>
                    <div class="pl-4">
                        <dl class="row">
                            <dt class="col-sm-3">Hazard category</dt>
                            <dd class="col-sm-9">
                                <@com.picklist carcinogenicity.HazardCategory/>
                            </dd>
                            <dt class="col-sm-3">Hazard statement</dt>
                            <dd class="col-sm-9">
                                <@com.picklist carcinogenicity.HazardStatement/>
                            </dd>
                            <dt class="col-sm-3">Route of exposure</dt>
                            <dd class="col-sm-9">
                                <p><@printMultilingualPicklistMultiple carcinogenicity.RouteExposure ""/></p>
                            </dd>
                        </dl>
                    </div>
                </#if>
            </#if>

			<#if (record.Classification.AdditionalHazard.Classes?has_content)>
				<h6>Additional hazard classes</h6>
                <div class="pl-4">
                    <dl class="row">
                        <dt class="col-sm-3"></dt>
                        <dd class="col-sm-9">
                            <#local additionalHazardClasses = record.Classification.AdditionalHazard.Classes/>
                            <@printMultilingualField additionalHazardClasses/>
                        </dd>
                    </dl>
                </div>
			</#if>

			<#if (record.Classification.AdditionalHazard.Statements?has_content)>
				<h6>Additional hazard statements</h6>
                <div class="pl-4">
                    <dl class="row">
                        <dt class="col-sm-3"></dt>
                        <dd class="col-sm-9">
                            <#local additionalHazardStatements = record.Classification.AdditionalHazard.Statements/>
                            <@printMultilingualField additionalHazardStatements/>
                        </dd>
                    </dl>
                </div>
			</#if>

			<#if isClassificationSpecificConcLimits(record) && isClassified>
				<h6>Specific concentration limits</h6>
                <div class="pl-4">
                    <dl class="row">
                        <#list record.Classification.SpecificConcentrations.ConcentrationRange as item>
                            <#if isNotEmptyConcentrationBlock(item)>
                                <dt class="col-sm-3">Concentration (%)</dt>
                                <dd class="col-sm-9 mb-0">
                                    <p><@com.range item.ConcentrationRangeVal/></p>
                                </dd>
                                <dt class="col-sm-3">Classification</dt>
                                <dd class="col-sm-9">
                                    <p><@com.picklistMultiple item.HazardCategories/></p>
                                </dd>
                            </#if>
                        </#list>
                    </dl>
                </div>
			</#if>
		</div>
	</#if>

	<#local physicalHazardClassifications = hazardsObjectToList(getPhysicalHazards(record))/>
	<#if (physicalHazardClassifications?size > 0) && isClassified>
		<div class="physicalClassification">
			<h5>Physical hazards</h5>
			<dl class="row">
				<#list physicalHazardClassifications as hazard>
					<dt class="col-sm-3">${hazard.type}</dt>
					<dd class="col-sm-9">
						<p><@com.picklist hazard.content.HazardCategory/>-<@com.picklist hazard.content.HazardStatement!/></p>
					</dd>
				</#list>
			</dl>
		</div>
	</#if>
    <#if (!healthHazardClassifications?has_content && !physicalHazardClassifications?has_content && isClassified)>
        <p>No health nor physical hazards described</p>
    </#if>
</#macro>

<#macro printEnvironmentalClassification ghsRecords>
    <#local envHazards = mergeEnvironmentalHazards(ghsRecords)/>
    <#if (!envHazards?has_content)>
        <#return/>
    </#if>
    <div class="environmentalHazards">
        <h5>Environmental hazards</h5>
        <dl class="row">
        <#if (envHazards.acuteShortTerm?size > 0)>
            <dt class="col-sm-3">Hazards to the aquatic environment (acute/short-term)</dt>
            <dd class="col-sm-9">
                <@listStrings removeDuplicates(envHazards.acuteShortTerm) true/>
            </dd>
        </#if>
        <#if (envHazards.longTerm?size > 0)>
            <dt class="col-sm-3">Hazards to the aquatic environment (chronic/long-term)</dt>
            <dd class="col-sm-9">
                <@listStrings removeDuplicates(envHazards.longTerm) true/>

            </dd>
        </#if>
        <#if (envHazards.mFactorAcute?size > 0)>
            <dt class="col-sm-3">M-Factor acute</dt>
            <dd class="col-sm-9">
                <@listStrings removeDuplicates(envHazards.mFactorAcute) true/>
            </dd>
        </#if>
        <#if (envHazards.mFactorChronic?size > 0)>
            <dt class="col-sm-3">M-Factor chronic</dt>
            <dd class="col-sm-9">
                <@listStrings removeDuplicates(envHazards.mFactorChronic) true/>
            </dd>
        </#if>
        <#if (envHazards.ozoneLayer?size > 0)>
            <dt class="col-sm-3">Hazardous to the ozone layer</dt>
            <dd class="col-sm-9">
                <@listStrings removeDuplicates(envHazards.ozoneLayer) true/>
            </dd>
        </#if>
        </dl>
    </div>
</#macro>

<#function getEnvironmentalHazards record>
    <#local environmentalHazards = record.Classification.EnvironmentalHazards />
        <#if isNotEmptyClassificationBlock(environmentalHazards.AquaticEnvironment.AcuteShortTerm)>
            <#local acuteShortTerm><@com.picklist environmentalHazards.AquaticEnvironment.AcuteShortTerm.HazardCategory/>-<@com.picklist environmentalHazards.AquaticEnvironment.AcuteShortTerm.HazardStatement!/></#local>
        </#if>
        <#if isNotEmptyClassificationBlock(environmentalHazards.AquaticEnvironment.LongTerm)>
            <#local longTerm><@com.picklist environmentalHazards.AquaticEnvironment.LongTerm.HazardCategory/>-<@com.picklist environmentalHazards.AquaticEnvironment.LongTerm.HazardStatement!/></#local>
        </#if>
        <#if environmentalHazards.AquaticEnvironment.MFactor.MFactorAcute?has_content>
            <#local mFactorAcute><@com.number environmentalHazards.AquaticEnvironment.MFactor.MFactorAcute/></#local>
        </#if>
        <#if environmentalHazards.AquaticEnvironment.MFactor.MFactorChronic?has_content>
            <#local mFactorChronic><@com.number environmentalHazards.AquaticEnvironment.MFactor.MFactorChronic/></#local>
        </#if>
        <#if isNotEmptyClassificationBlock(environmentalHazards.OzoneLayer.HazardousOzone)>
            <#local ozoneLayer><@com.picklist environmentalHazards.OzoneLayer.HazardousOzone.HazardCategory/>-<@com.picklist environmentalHazards.OzoneLayer.HazardousOzone.HazardStatement!/></#local>
        </#if>

    <#return {"acuteShortTerm": acuteShortTerm!"", "longTerm": longTerm!"",
                "mFactorAcute": mFactorAcute!"", "mFactorChronic": mFactorChronic!"",
                "ozoneLayer": ozoneLayer!"" }>
</#function>

<#function mergeEnvironmentalHazards ghsRecords>
    <#local envHazardsList = []/>
    <#list ghsRecords as ghs>
        <#local isClassified = !ghs.GeneralInformation.NotClassified />
	    <#if isClassificationEnvironmentalHazards(ghs) && isClassified>
            <#local envHazards = getEnvironmentalHazards(ghs)>
            <#local envHazardsList = [envHazards] + envHazardsList>
        </#if>
    </#list>
    <#local envHazardsList = removeDuplicatedObjectsInList(envHazardsList)?reverse/>
    <#return mergeObjectValuesFromObjectList(envHazardsList)/>
</#function>

<#macro cnlNotes record>
    <#local noneSelected = true/> <#-- It can be that the note is added but not value selected in IUCLID  -->
    <#local isNA = false/>
	<#if (record.NotesBlock.Notes?has_content)>
        <#list record.NotesBlock.Notes as note>
            <#if (note?has_content)>
                <#local localizedPhrase = iuclid.localizedPhraseDefinitionFor(note.Note.code, "en") />
                <dt class="col-sm-3">${localizedPhrase.text}</dt>
                <dd class="col-sm-9">${localizedPhrase.description}</dd>
                <#local noneSelected = false/>
            </#if>
        </#list>
    <#else>
        <#local isNA = true/>
    </#if>

    <#if noneSelected>
        <dt class="font-weight-normal col-12">None selected</dt><dd></dd>
	<#elseif isNA>
		<dt class="font-weight-normal col-12">NA</dt><dd></dd>
	</#if>
</#macro>
<#function buildMixtureReferences referenceList>
    <#local result = []>
    <#list referenceList as composition>
        <#local index = mixtureCompositionsIndex[composition]!""/>
        <#local ref><a href="#mixtureComp-${index}" onclick="goToMixture(${index})">Mixture #${index}</a> </#local>
        <#local result = [ref] + result/>
    </#list>
    <#return result?reverse>
</#function>

<#function getPhInformation mixture>
	<#local phSummaries = iuclid.getSectionDocumentsForParentKey(mixture.documentKey, "ENDPOINT_SUMMARY", "pH")/>
	<#local result = []/>
	<#if phSummaries?has_content>
		<#list phSummaries as summary>
            <#local mixtures = [] />
            <#local ph = ""/>
            <#local concentration = ""/>
            <#local concentration = ""/>
            <#local justification = ""/>
            <#local mixtures = []/>
			<#--  Valid dossiers only have one phSummary document  -->
			<#if summary?has_content>
                <#if summary.KeyValueForChemicalSafetyAssessment.pH?has_content>
                    <#local ph>
                            <@com.range summary.KeyValueForChemicalSafetyAssessment.pH />
                    </#local>
                <#else>
                    <#local ph>N/A</#local>
                </#if>
                <#if summary.KeyValueForChemicalSafetyAssessment.SolutionConcentration?has_content>
                    <#local concentration>
						<@com.number summary.KeyValueForChemicalSafetyAssessment.SolutionConcentration />
                    </#local>
				<#else>
                    <#local concentration>N/A</#local>
                </#if>

                <#if summary.KeyValueForChemicalSafetyAssessment.pHNotRelevant>
    				<#local justification>
						<@com.picklist summary.KeyValueForChemicalSafetyAssessment.Justification />
                    </#local>
                </#if>

                <#if (summary.Discussion.RelatedComposition?has_content)>
                    <#local mixtures = buildMixtureReferences(summary.Discussion.RelatedComposition)/>
                </#if>
    		</#if>
            <#if (ph == "")>
    			<#local ph = "Value not provided"/>
        	</#if>
	        <#if (concentration == "")>
		        <#local concentration = "Value not provided"/>
	        </#if>
        	<#local result = [{"ph": ph, "concentration": concentration, "justification": justification, "mixtures": mixtures}] + result />
		</#list>
	</#if>
	<#return result?reverse/>
</#function>

<#function getProductContacts mixture>
<#--  Basically all the contacts but the emergency contact  -->
	<#local contacts = []/>
	<#if mixture.ContactPersons?has_content>
		<#list mixture.ContactPersons as contact>
			<#if contact?has_content>
				<#local c = iuclid.getDocumentForKey(contact.ContactPerson)  + {"key": contact.ContactPerson?keep_before("/")}/>
				<#if c?has_content>
					<#if !com.picklistValueMatchesPhrases(c.GeneralInfo.ContactType, ["emergency contact"])>
						<#local contacts = [c] + contacts/>
					</#if>
				</#if>
			</#if>
		</#list>
	</#if>
	<#return contacts>
</#function>

<#function getEmergencyContacts mixture>
	<#local contacts = []/>
	<#if mixture.ContactPersons?has_content>
		<#list mixture.ContactPersons as contact>
			<#if contact?has_content>
				<#local c = iuclid.getDocumentForKey(contact.ContactPerson) + {"key": contact.ContactPerson?keep_before("/")}/>
				<#if c?has_content>
					<#if com.picklistValueMatchesPhrases(c.GeneralInfo.ContactType, ["emergency contact"])>
						<#local contacts = [c] + contacts/>
					</#if>
				</#if>
			</#if>
		</#list>
	</#if>
	<#return contacts>
</#function>

<#function getCountryName country>
	<#local splitCharacter = country?index_of('[') - 1 />
	<#if (splitCharacter < 0)>
		<#return country/>
	<#else>
		<#return country[0..splitCharacter] />
	</#if>
</#function>

<#function getCountryCode country>
    <#if (country == "")>
        <#return "NA"/>
    </#if>
	<#local splitCharacter = country?index_of('[') - 1 />
	<#if (splitCharacter < 0)>
		<#return country/>
	<#else>
		<#return country[splitCharacter..] />
	</#if>
</#function>

<#macro printContactHeader contact contactId>
	<a class="font-weight-normal collapsed" data-toggle="collapse" data-target="#${contactId}">
		<i class="toggle fa"></i>
			<#if (contact.GeneralInfo.Country?has_content)>
				<#local country><@com.picklist contact.GeneralInfo.Country!"" /></#local>
				<#local countryCode = getCountryCode(country)/>
				<#local countryName = getCountryName(country)/>
				${countryCode} <@com.text contact.GeneralInfo.LastName /><#if contact.GeneralInfo.FirstName?has_content>, <@com.text contact.GeneralInfo.FirstName /></#if>: <@com.text contact.GeneralInfo.Organisation /> - ${countryName}
			<#else>
				<@com.text contact.GeneralInfo.LastName /><#if contact.GeneralInfo.FirstName?has_content>, <@com.text contact.GeneralInfo.FirstName /></#if>: <@com.text contact.GeneralInfo.Organisation />
			</#if>
	</a>
</#macro>

<#macro printContactBody contact>
	<#if contact.GeneralInfo.Phone?has_content>
		<p>Tel: <@com.text contact.GeneralInfo.Phone /></p>
	<#else>
		<p>Tel: Not provided</p>
	</#if>
	<#if contact.GeneralInfo.Email?has_content>
		<p>Email: <@com.text contact.GeneralInfo.Email /></p>
	<#else>
		<p>Email: Not provided</p>
	</#if>
</#macro>

<#macro printContacts contacts idPrefix>
	<#if !(contacts?has_content)>
		NA
		<#return/>
	</#if>

	<dl>
	<#local contactCounter = 1/>
	<#list contacts as contact>
		<#local contactId = idPrefix + contactCounter/>
		<dt>
			<@printContactHeader contact contactId/>
		</dt>
		<dd id="${contactId}" class="collapse d-print-block" >
			<@printContactBody contact/>
		</dd>
		<#local contactCounter = contactCounter + 1/>
	</#list>
	</dl>
</#macro>

<#function getComponentSummary c>
    <#local component = iuclid.getDocumentForKey(c.Reference)/>
    <#local name>
        <#if (component.documentType?has_content && component.documentType=="MIXTURE")>
            <@com.text component.MixtureName />
        </#if>
        <#if (component.documentType?has_content &&  component.documentType=="SUBSTANCE")>
            <@com.text component.ChemicalName />
        </#if>
    </#local>
    <#local concentration>
        <#if (c.TypicalConcentration.lower.value)?has_content>
            <@com.range c.TypicalConcentration/>
        <#elseif c.ConcentrationRange?has_content>
            <@com.range c.ConcentrationRange />
        </#if>
    </#local>
    <#local componentSummary = {"name": name, "concentration": concentration}/>
    <#return componentSummary>
</#function>

<#function isInCommonComponentList component commonComponentList>
    <#local componentSummary = getComponentSummary(component)/>
    <#return isObjectInList(componentSummary commonComponentList)>
</#function>

<#function getCommonComponents mixtureCompositionList>
    <#local numberOfMixtures = mixtureCompositionList?size/>
    <#local joinedListOfComponents = []/>
    <#--  Let's put all the components from all the mixtures in the same list  -->
    <#list mixtureCompositionList as composition>
        <#list composition.Components.Components as c>
            <#local componentSummary = getComponentSummary(c)/>
            <#local joinedListOfComponents = [componentSummary] + joinedListOfComponents/>
        </#list>
    </#list>
    <#local accumulatorObject = {}/>
    <#local commonComponents = []/>
    <#--  A common component is the one that appears as many times as there mixtures  -->
    <#list joinedListOfComponents as component>
        <#local componentKey = createHash(component)/>
        <#if (accumulatorObject[componentKey]?has_content)>
            <#-- That complex way of building objects/hashes and modifying its content    -->
            <#local accumulatorObject = accumulatorObject + {componentKey: accumulatorObject[componentKey] + 1}/>
        <#else>
            <#local accumulatorObject = accumulatorObject + {componentKey: 1}/>
        </#if>
        <#if (accumulatorObject[componentKey] == numberOfMixtures)>
            <#local commonComponents = [component] + commonComponents/>
        </#if>
    </#list>
    <#return commonComponents?reverse/>
</#function>

<#function getComponentInfo c>
	<#local component = iuclid.getDocumentForKey(c.Reference)/>

	<#local name = "NA"/>
	<#local key = (component.documentKey)!"" />
	<#local func>
		<#if c.Function?has_content>
			<@printMultilingualPicklist c.Function/>
		</#if>
	</#local>
	<#local identifiers = {}/>
	<#local components = []/>
	<#local suppliers = []/>
	<#local legalEntity = ""/>
	<#local concentration>
		<#if c.TypicalConcentration.lower.value?has_content>
			<@com.range c.TypicalConcentration/>
		<#elseif c.ConcentrationRange?has_content>
			<@com.range c.ConcentrationRange />
		</#if>
	</#local>
	<#local signalWord = ""/>
	<#local pictograms = ""/>
	<#local type = component.documentType!""/>
	<#local displayType = ""/>
	<#local classification = []/>
	<#local hazardStatements = []/>
	<#local precautionaryStatements = []/>
	<#if (component.documentKey?has_content)>
		<#local ghsRecs=iuclid.getSectionDocumentsForParentKey(component.documentKey, "FLEXIBLE_RECORD", "Ghs" ) />
		<#--  There should be only one record in a PCN notification  -->
        <#if (ghsRecs?has_content)>
            <#local record = ghsRecs[0]/>
            <#if record.Labelling.SignalWord?has_content>
                <#local signalWord><@com.picklist record.Labelling.SignalWord /></#local>
            </#if>

            <#if record.Labelling.HazardPictogramBlock.HazardPictogram?has_content>
                <#local pictograms><@HazardPictogramList record.Labelling.HazardPictogramBlock.HazardPictogram/></#local>
            </#if>
            <#local classification = getCompactClassification(record)/>
            <#if record.Labelling.HazardStatementsBlock.HazardStatements?has_content>
                <#local hazardStatements = record.Labelling.HazardStatementsBlock.HazardStatements/>
            </#if>
            <#if record.Labelling.PrecautionaryStatementsBlock.PrecautionaryStatements?has_content>
                <#local precautionaryStatements = record.Labelling.PrecautionaryStatementsBlock.PrecautionaryStatements/>
            </#if>
        </#if>
    </#if>
	<#local legalEntity = ""/>
	<#if (component.documentType?has_content && component.documentType!="REFERENCE_SUBSTANCE")>
		<#local ownerLegalEntity = iuclid.getDocumentForKey(component.OwnerLegalEntity)/>
        <#--  Check added due to https://dev.azure.com/echa-ecm/ECOMOD/_workitems/edit/651605 , it is incorrect format but it was allowed at some point  -->
        <#if ownerLegalEntity?has_content && ownerLegalEntity.documentType == "LEGAL_ENTITY" && (ownerLegalEntity.GeneralInfo?has_content)>
            <#local legalEntity><@printLegalEntity ownerLegalEntity "owner"/></#local>

		<#else>
			<#-- We only report on the first LE mentioned in the supplier list   -->
            <#local legalEntity=""/>
            <#--  TODO: Fix below as it fails for SUPPLIERS document  -->
            <#--  <#local ownerLegalEntity = iuclid.getDocumentForKey(ownerLegalEntity[0].ManufacturerImportForm.LegalEntity)/>  -->
		</#if>
	</#if>

	<#if (component.documentType?has_content && component.documentType=="MIXTURE")>
		<#local otherIdentifiers = otherNames(component)/>
		<#local phInfoList = getPhInformation(component)/>
		<#local sdsList = pcn.getSdsList(component)/>
		<#local supplierRecords = iuclid.getSectionDocumentsForParentKey(component.documentKey, "FLEXIBLE_RECORD", "Suppliers")/>
		<#local name><@com.text component.MixtureName /></#local>
		<#local ufis = removeDuplicates(getRegulatoryIdentifiers(component, ".*CLP unique formula identifier.*"))/>
		<#local identifiers = {"UFI": ufis}/>
		<#local displayType = "flask"/>
		<#local type = "Mixture"/>

		<#if (supplierRecords?has_content)>
			<#list supplierRecords as supplier>
				<#local supplierLegalEntity = iuclid.getDocumentForKey(supplier.ManufacturerImportForm.LegalEntity)/>
				<#if supplierLegalEntity?has_content>
					<#local s><@printLegalEntity supplierLegalEntity supplier.ManufacturerImportForm.LegalEntity?keep_before("/") /></#local>
					<#local suppliers = [s] + suppliers/>
				</#if>
			</#list>
		</#if>
		<#local compositionList = iuclid.getSectionDocumentsForParentKey(component.documentKey, "FLEXIBLE_RECORD", "MixtureComposition") />
		<#list compositionList as composition>
			<#local components = components + addSortValue(composition.Components.Components) />
		</#list>
	</#if>

	<#if (component.documentType?has_content && component.documentType=="SUBSTANCE")>
		<#local name><@com.text component.ChemicalName /></#local>
		<#local refSubst = iuclid.getDocumentForKey(component.ReferenceSubstance.ReferenceSubstance)/>
		<#local identifiers = substanceIdentifiers(refSubst)/>
		<#local displayType = "vial"/>
		<#local type = "Substance"/>
	</#if>

	<#if (c.Icg)>
		<#local displayType = "recycle"/>
		<#local type = type + " ICG"/>
	</#if>

	<#if (c.Gci)>
		<#if (func == "colourant")>
			<#local displayType = "palette"/>
			<#local type = type + " Colouring agent"/>
		<#elseif (func == "perfume")>
			<#local displayType = "wind"/>
			<#local type = type + " Perfume"/>
		</#if>
	</#if>

	<#if (c.Sfc)>
		<#local displayType = "book"/>
		<#local type =  type + " - standard formula component"/>
	</#if>

	<#return {"ghsRecords": ghsRecs![], "name": name, "key": key, "func": func,"identifiers": identifiers, "otherIdentifiers": otherIdentifiers![], "components": sortByConcentration(components), "referenceSubstance": refSubst!"", "suppliers": suppliers?reverse, "legalEntity": legalEntity, "concentration": concentration, "phInfoList": (phInfoList)![], "signalWord": signalWord, "pictograms": pictograms, "displayType": displayType, "type": type, "classification": classification, "hazardStatements": hazardStatements, "precautionaryStatements": precautionaryStatements, "sdsList": sdsList!{"sdsSection11List": [], "sdsAttachmentList": []} }/>
</#function>

<#macro cleanId id>
<#compress>
    ${id?remove_beginning("M")}
</#compress>
</#macro>

<#macro printIconForNameType type>
    <#if type?contains("standard formula (SF)")>
        <i class="fa fa-book" title="Standard formula"></i>
    <#elseif type?contains("fuels")>
        <i class="fa fa-gas-pump" title="Fuels"></i>
    <#elseif type?contains("common name")>
        <i class="fa fa-tag" title="Common name"></i>
    </#if>
</#macro>

<#macro printComponentInfo c id level visited>

	<tr>
		<td>
			<#if (c.key == "")> <#-- Component has not entity associated -->
				<i class="fa fa-${c.displayType} text-dark" title="${c.type}"></i>
				<@cleanId id/>: ${c.name}
			<#elseif visited >
				<i class="fa fa-${c.displayType} text-dark" title="${c.type}"></i>
				<i class="toggle fa"></i>
				<@cleanId id/>: ${c.name} (mixture repeated and already displayed above)
				<p><@printIdentifiersForComponent c.identifiers false true/></p>
			<#elseif (level < maxLevelsDisplayed)>
				<a class="font-weight-bold collapsed" data-toggle="collapse" data-target="#component-${id}">
					<i class="fa fa-${c.displayType} text-dark" title="${c.type}"></i>
					<i class="toggle fa"></i>
					<@cleanId id/>: ${c.name}

				</a>
				<p><@printIdentifiersForComponent c.identifiers false true/></p>
			<#else>
				<a class="font-weight-bold">
					<i class="fa fa-${c.displayType} text-dark" title="${c.type}"></i>
					${c.name}
				</a>
				<p><@printIdentifiersForComponent c.identifiers false true/></p>
			</#if>
		</td>
		<td>${c.func}</td>
		<td>${c.concentration}</td>
		<td>
			<@listStrings removeHazardStatements(c.classification) false true/>
		</td>
		<td class="no-wrap">
			<span class="badge badge-pill badge-${c.signalWord?lower_case}">${c.signalWord}</span>
			${c.pictograms}
		</td>
	</tr>
</#macro>
<#function getLanguageName code>
	<#local languages = {
		'bg': 'Bulgarian',
		'ga': 'Irish',
		'hr': 'Croatian',
		'cs': 'Czech',
		'da': 'Danish',
		'nl': 'Dutch',
		'en': 'English',
		'et': 'Estonian',
		'fi': 'Finnish',
		'fr': 'French',
		'de': 'German',
		'el': 'Greek',
		'hu': 'Hungarian',
		'is': 'Icelandic',
		'it': 'Italian',
		'la': 'Latvian', <!-- code for Latin but found in prod dossiers for Latvian -->
		'lv': 'Latvian',
		'lt': 'Lithuanian',
		'mt': 'Maltese',
		'no': 'Norwegian',
		'pl': 'Polish',
		'pt': 'Portuguese',
		'ro': 'Romanian',
		'sk': 'Slovak',
		'sl': 'Slovenian',
		'es': 'Spanish',
		'sv': 'Swedish'
	} />
	<#return languages[code]!"default"/>
</#function>

<#macro printToxInfo sdsList id bgColour>
	<#if (id !="main")>
		<div class="row">
			<div class="col-12">
				<h5 class="rounded bg-${bgColour} text-white">Toxicological information
					<button class="btn btn-${bgColour} collapsed" data-toggle="collapse" title="Toggle display of Toxicological information for component" data-target="#component-tox-${id}">
						<i class="toggle fa"></i>
					</button>
				</h5>
			</div>
		</div>
	</#if>
	<#if (id == "main")>
		<div class="row">
		<#local textColour></#local>
	<#else>
		<div class="row ml-2 collapse d-print-flex"  id="component-tox-${id}">
		<#local textColour>text-white</#local>
	</#if>
	<#if (sdsList.sdsSection11List?size > 0 && sdsList.sdsSection11List[0].text !="")>
		<#local toxInfo = sdsList.sdsSection11List/>
        <#if (id != "main")>
            <div class="col-12">
	    		<h6 class="rounded bg-${bgColour} ${textColour}">Section 11 of the SDS</h6>
    		</div>
        </#if>
		<div class="col d-print-none">
			<select title="Select language for Section 11 of the safety data sheet" name="section11Lang" id="section11Lang-select-${id}" onchange="changeText(this, '${id}');">
				<option value="">-- Select language --</option>
                <#local languagesList = []/>
				<#list toxInfo as section11>
					<#if (section11.language?has_content)>
                         <#local lang = section11.language/>
                         <#--  Avoid repeated languages  -->
                         <#if (!languagesList?seq_contains(lang))>
                            <#local languagesList = [lang] + languagesList/>
                            <option value="${lang}">${section11.languageName} - [${lang}]</option>
                         </#if>
					</#if>
				</#list>
			</select>
		</div>
		<div class="col-12">
			<#list toxInfo as section11>
				<#if (section11.text !="")>
					<div class="sdsSec11 tox-${id} active d-print-block" lang="${section11.language}">
						<p class="badge badge-secondary">${section11.language!"default"}</p>
                        <#if (id == "main")>
                            <#if (section11.mixtures?size > 0)>
                                <@listStrings section11.mixtures false true/>
                            </#if>
                        </#if>
						<div class="sdsSec11Text border">
							${section11.text}
						</div>
					</div>
				</#if>
			</#list>
		</div>
	<#else>
        <div class="col-12">
		    <p>NA</p>
        </div>
	</#if>
	</div>
</#macro>

<#macro printComponentSDSInfo c id bgColour>
	<div class="row">
		<div class="col-12">
			<h5 class="rounded bg-${bgColour} text-white">Safety data sheet
				<button class="btn btn-${bgColour} collapsed" data-toggle="collapse"
                title="Toggle display of safety data sheet for component" data-target="#component-sds-${id}">
					<i class="toggle fa"></i>
				</button>
			</h5>
		</div>
	</div>
	<div class="row ml-2 collapse d-print-flex"  id="component-sds-${id}">
		<#if (c.sdsList.sdsAttachmentList?size > 0)>
			<div class="col-12">
				<#list c.sdsList.sdsAttachmentList?sort_by("countryCode") as attachment>
					<#assign attachmentText>${attachment.countryCode} ${(attachment.filename)} <span class="badge badge-secondary">${attachment.language}</span></#assign>
					<#if (mode == "PUBLISHING")>
						<p class="ml-2"><a href="${attachment.uri}" target="_blank">${attachmentText} <i class="fas fa-download"></i></a></p>
					<#else>
						<p class="ml-2">${attachmentText}</p>
					</#if>
				</#list>
			</div>
		<#else>
			NA
		</#if>
	</div>
</#macro>


<#macro printComponentPhInfo c id bgColour>
	<#assign phInfoList = c.phInfoList/>
	<div class="row">
		<div class="col-12">
			<h5 class="rounded bg-${bgColour} text-white">pH information
				<button class="btn btn-${bgColour} collapsed" data-toggle="collapse"
                title="Toggle display of pH information for component" data-target="#component-ph-${id}">
					<i class="toggle fa"></i>
				</button>
			</h5>
		</div>
	</div>
	<div class="row ml-2 collapse d-print-flex"  id="component-ph-${id}">
		<#if (!phInfoList?has_content)>
			<div class="col-12">
				pH information not available
			</div>
			<#return>
		</#if>
		<div class="col-12">
            <@printPhInformation phInfoList/>
		</div>
	</div>
</#macro>

<#macro printPhInformation phInfoList main=false>
    <table class="phTable table">
        <thead>
            <tr>
                <th>pH value</th>
                <th>Solution concentration</th>
                <th>Justification</th>
                <#--  We don't print mixture links if we are not in the main tab
                or there are no mixtures linked in the phInfoSummaries  -->
                <#if (main) && (phInfoList?filter(x -> x.mixtures?size > 0)?size > 0)>
                    <th>Mixture compositions</th>
                </#if>
            </tr>
        </thead>
        <tbody>
            <#list phInfoList as phInfo>
                <tr>
                    <#if (phInfo.justification != "")>
                        <td><p>Not relevant</p></td>
                        <td><p>Not relevant</p></td>
                        <td><p>${phInfo.justification}</p></td>
                    <#else>
                        <td><p>${phInfo.ph}</p></td>
                        <td><p>${phInfo.concentration}</p></td>
                        <td><p>—</p></td>
                    </#if>
                    <#if (main) && (phInfo.mixtures?size > 0)>
                        <td><@listStrings phInfo.mixtures true true/></td>
                    </#if>
                </tr>
            </#list>
        </tbody>
    </table>

</#macro>

<#function getAvailableLanguages section11>
	<#assign result = []/>
	<#list section11 as element>
		<#assign result = [element.language] + result/>
	</#list>
	<#return result?reverse>
</#function>

<#macro printComponentClassification c id bgColour>
	<div class="row">
		<div class="col-12">
			<h5 class="rounded bg-${bgColour} text-white">Classification and Labelling
				<button class="btn btn-${bgColour} collapsed" data-toggle="collapse"
                title="Toggle display of classification and labelling information for component" data-target="#component-classification-${id}">
					<i class="toggle fa"></i>
				</button>
			</h5>
		</div>
	</div>

	<div class="row ml-2 collapse d-print-flex"  id="component-classification-${id}">
		<#if (c.ghsRecords?size == 0)>
			NA
		<#else>
			<div class="col-12">
                <#-- Components should only have 1 ghs record, if more we only display the first one    -->
                <#local ghs = c.ghsRecords[0]>
                <h6 class="rounded bg-${bgColour} text-white">Classification</h6>
                <div class="row">
                    <div class="col-12">
                        <#if (ghs.GeneralInformation.NotClassified)>
                            <p>Not classified</p>
                        </#if>
                        <@ghsClassification ghs/>
                        <@printEnvironmentalClassification [ghs]/>
                    </div>
                </div>
                <h6 class="rounded bg-${bgColour} text-white">Labelling</h6>
                <div class="row">
                    <div class="col-12">
                        <dl class="row">
                            <@printLabelling ghs/>
                        </dl>
                    </div>
                </div>
                <h6 class="rounded bg-${bgColour} text-white">Additional labelling requirements</h6>
                <div class="row">
                    <div class="col-12">
                        <dl class="row">
                            <@printAdditionalLabelling ghs/>
                        </dl>
                    </div>
                </div>
                <h6 class="rounded bg-${bgColour} text-white">Notes</h6>
                <div class="row">
                    <div class="col-12">
                        <dl class="row">
                            <@cnlNotes ghs/>
                        </dl>
                    </div>
                </div>
			</div>
		</#if>
	</div>
</#macro>
<#macro printComponentComposition component id level bgColour>
	<div class="row">
		<div class="col-12">
			<h5 class="rounded bg-${bgColour} text-white">Composition</h5>
		</div>
		</div>
		<div class="row">
		<div class="col-12 ml-2">
			<table class="table">
				<thead>
					<tr>
						<th>Name</th>
						<th>Function</th>
						<th>Concentration (w/w)</th>
						<th>Classification</th>
						<th>Labelling</th>
					</tr>
				</thead>
				<tbody>
					<#local subComponentId = 1>
					<#list component.components as c>
						<#local stringId><@cleanId id/>-${subComponentId}</#local>
						<@printComponent c stringId level/>
						<#local subComponentId = subComponentId + 1/>
					</#list>
				</tbody>
			</table>
		</div>
	</div>
</#macro>

<#macro printOtherIdentifiers identifiers>
	<table class="table">
		<thead>
			<tr>
				<th>Name type</th>
				<th>Identifier</th>
				<th>Countries</th>
				<th>Remarks</th>
			</tr>
		</thead>
		<tbody>
			<#list identifiers as id>
				<td>${id.type}</td>
				<td>${id.identifier!""}</td>
				<td>${id.countries}</td>
				<td><@printMultilingualField (id.remarks)!{} false false/></td>
			</#list>
		</tbody>
	</table>
</#macro>

<#function addVisitedMixture c id>
	<#return ([{"key":c.key, "id": id}] + visitedComponents)>
</#function>

<#function isComponentVisited c id>
	<#list visitedComponents as element>
		<#if (element.key == c.key && id?starts_with(element.id))>
			<#return true/>
		</#if>
	</#list>
	<#return false>
</#function>

<#macro printComponent c stringId level>
	<#local cInfo = getComponentInfo(c.sortedComponent)/>
	<#local nextLevel = level + 1/>
	<#--  In case component does not refer to any mixture or substance we only display the basic info  -->
	<#if (cInfo.key == "")>
		<#assign visited = false />

		<@printComponentInfo cInfo stringId nextLevel visited/>
		<#return />
	</#if>

	<#assign visited = isComponentVisited(cInfo, stringId) />
	<#if (!visited)>
		<#if (cInfo.type?starts_with("Mixture"))>
			<#assign visitedComponents = addVisitedMixture(cInfo, stringId)/>
		</#if>
		<@printComponentInfo cInfo stringId nextLevel visited/>
		<#if (level < maxLevelsDisplayed)>
			<@printComponentDetails cInfo stringId nextLevel/>
		</#if>
	<#else>
		<@printComponentInfo cInfo stringId nextLevel visited/>
	</#if>
</#macro>

<#macro printComponentIdentification componentInfo id bgColour>

	<div class="row">
		<div class="col-12">
			<h5 class="rounded bg-${bgColour} text-white">Identification
				<button class="btn btn-${bgColour} collapsed" data-toggle="collapse"
                title="Toggle display of Toxicological information for component" data-target="#component-identification-${id}">
					<i class="toggle fa"></i>
				</button>
			</h5>
		</div>
	</div>
	<div class="row ml-2 collapse d-print-flex"  id="component-identification-${id}">
        <div class="col-12">
            <#if (componentInfo.type?starts_with("Substance"))>
                <dl class="row">
                    <dt class="col-sm-2">Reference substance</dt>
                    <dd class="col-sm-4">
                        <@com.text componentInfo.referenceSubstance.ReferenceSubstanceName!""/>
                    </dd>

                    <dt class="col-sm-2">Legal entity</dt>
                    <dd class="col-sm-4">
                        <#if (componentInfo.legalEntity?has_content)>
                            ${componentInfo.legalEntity}
                        <#else>
                        NA
                        </#if>
                    </dd>
                    <#if (componentInfo.identifiers["IUPAC"]?has_content)>
                                <dt class="col-sm-2">IUPAC name</dt>
                                <dd class="col-sm-4">
                                    ${componentInfo.identifiers["IUPAC"]}
                                </dd>
                    </#if>
                    <#if (componentInfo.identifiers["Synonyms"]?has_content)>
                        <dt class="col-sm-2">Synonyms</dt>
                        <dd class="col-sm-4">
                            <@listStrings removeDuplicates(componentInfo.identifiers["Synonyms"]) true true/>
                        </dd>
                    </#if>
                    <#if (componentInfo.identifiers["CAS no"]?has_content)>
                        <dt class="col-sm-2">CAS no</dt>
                        <dd class="col-sm-4">
                            ${componentInfo.identifiers["CAS no"]}
                        </dd>
                    </#if>
                    <#if (componentInfo.identifiers["Molecular Formula"]?has_content)>
                        <dt class="col-sm-2">Molecular formula</dt>
                        <dd class="col-sm-4">
                            ${componentInfo.identifiers["Molecular Formula"]}
                        </dd>
                    </#if>
                    <#if (componentInfo.identifiers["EC no"]?has_content)>
                        <dt class="col-sm-2">EC no</dt>
                        <dd class="col-sm-4">
                            ${componentInfo.identifiers["EC no"]}
                        </dd>
                    </#if>
                </dl>
            </#if>
            <#if (componentInfo.type?starts_with("Mixture") && !(componentInfo.type?contains("standard formula component") || componentInfo.type?contains("ICG")))>
                <dl class="row">
                    <dt class="col-sm-2">UFI(s)</dt>
                    <dd class="col-sm-4">
                        <@listStrings componentInfo.identifiers["UFI"] true/>
                    </dd>
                    <#list componentInfo.suppliers as supplier>
                        <dt class="col-sm-2">Supplier</dt>
                        <dd class="col-sm-4">
                            ${supplier}
                        </dd>
                    <#else>
                        <dt class="col-sm-2">Supplier</dt>
                        <dd class="col-sm-4">
                            <p>No supplier provided</p>
                        </dd>
                    </#list>
                </dl>
            <#elseif (componentInfo.type?contains("standard formula component") || componentInfo.type?contains("ICG"))>
                <dl class="row">
                    <dt class="col-sm-2">Supplier</dt>
                    <dd class="col-sm-4">
                        <#--  <p><@listStrings componentInfo.suppliers/></p>  -->
                        <p>${componentInfo.legalEntity}</p>
                    </dd>
                </dl>
                <#if (componentInfo.otherIdentifiers?size > 0)>
                    <div class="row">
                        <@printOtherIdentifiers componentInfo.otherIdentifiers/>
                    </div>
                </#if>
            </#if>
        </div>
    </div>
</#macro>

<#macro printComponentDetails componentInfo id level>
	<#local bgColour>
		<#if (level < 2)>
			secondary
		<#else>
			info
		</#if>
	</#local>
	<#local boxColour>
		<#if (level < 2)>
			bg-light
		<#else>
			mim-box
		</#if>
	</#local>

	<tr id="component-${id}" class="collapse border ${boxColour} d-print-table-row">
		<td colspan="6">
			<div class="container-fluid">
				<!-- START CompositionComponentDetails -->
				<div class="row">
					<div class="col-12">
						<h5>Component <@cleanId id/>: ${componentInfo.name}</h5>
					</div>
				</div>
				<#if (componentInfo.type?starts_with("Mixture"))>
					<@printComponentComposition componentInfo id level bgColour?trim/>
				</#if>
				<@printComponentIdentification componentInfo id bgColour?trim/>
				<@printComponentClassification componentInfo id bgColour?trim/>
				<#if (componentInfo.type?starts_with("Mixture") && !componentInfo.type?ends_with("ICG"))>
					<@printComponentSDSInfo componentInfo id bgColour?trim/>
				</#if>
				<#if (componentInfo.type?ends_with("ICG"))>
					<@printToxInfo componentInfo.sdsList id bgColour?trim/>
					<@printComponentPhInfo componentInfo id bgColour?trim/>
				</#if>
			</div>
			<!-- END CompositionComponentDetails -->
		</td>
	</tr>
</#macro>

<#macro printLegalEntity legalEntity id>
    <#if !(legalEntity.GeneralInfo?has_content)>
        No legal entity provided
        <#return/>
    </#if>

    <#if (legalEntity.GeneralInfo.ContactAddress?has_content)>
        <#local country><@com.picklist legalEntity.GeneralInfo.ContactAddress.Country /></#local>
        <#if (country!="")>
            <#local country><@com.picklist legalEntity.GeneralInfo.ContactAddress.Country /></#local>
            <#local countryCode=getCountryCode(country)/>
        <#else>
            <#local country>Not provided</#local>
            <#local countryCode=""/>
        </#if>

        <a class="font-weight-normal collapsed" data-toggle="collapse" data-target="#LE-${id}">
            <i class="toggle fa"></i>
            ${countryCode}<@com.text legalEntity.GeneralInfo.LegalEntityName/>
        </a>
        <div id="LE-${id}" class="collapse d-print-block">
            <#if legalEntity.GeneralInfo.ContactAddress.Phone?has_content >
                <p>Tel: <@com.text legalEntity.GeneralInfo.ContactAddress.Phone/></p>
            <#else>
                <p>Tel: Not provided</p>
            </#if>
            <#if legalEntity.GeneralInfo.ContactAddress.Email?has_content >
                <p>Email: <@com.text legalEntity.GeneralInfo.ContactAddress.Email/></p>
            <#else>
                <p>Email: Not provided</p>
            </#if>
            <p>Country: ${getCountryName(country)}</p>
        </div>
    <#else>
        <a class="font-weight-normal collapsed" data-toggle="collapse" data-target="#LE-${id}">
        <i class="toggle fa"></i>
            <@com.text legalEntity.GeneralInfo.LegalEntityName/>
        </a>
        <div id="LE-${id}" class="collapse d-print-block">
                No contact info provided
        </div>
    </#if>
</#macro>

<#macro printMultilingualField field isRemarks=false printNA=true>
    <#if (field?is_string && !field?is_hash)>
        <#if (field != "")>
            ${field}
    	<#elseif printNA>
		    NA
        </#if>
        <#return/>
    </#if>

	<#if (field.localizedTexts)?has_content>
		<#if (isRemarks)>
			<br/><b>Remarks:</b><br/>
		</#if>
		<#list field.localizedTexts?sort_by("language") as localizedText>
            <#if (localizedText.value?has_content)>
            <p class="mlField" lang="<@com.text localizedText.language/>">
                <span class="badge badge-secondary">
                    <@com.text localizedText.language/>
                </span>
                <@com.text localizedText.value/>
            </p>
            <#elseif printNA>
                <p class="mlField" lang="<@com.text localizedText.language/>"><span class="badge badge-secondary"><@com.text localizedText.language/></span>NA</p>
            </#if>
		</#list>
	<#elseif field?has_content> <#-- Not multilingual (old dossier) -->
		<p><span class="badge badge-secondary">default</span>
			<@com.text field/>
		</p>
	<#elseif printNA>
		NA
	</#if>
</#macro>

<#macro printMultilingualPicklist picklistValue printRemarks=false printNA=true locale="en" >
	<#local localizedPhrase = iuclid.localizedPhraseDefinitionFor(picklistValue.code, locale) />
	<#if localizedPhrase?has_content>
		${localizedPhrase.text} <#t>
		<#if localizedPhrase.description?has_content>
				[${localizedPhrase.description}]
		</#if>
		<#if localizedPhrase.open>
		<#--  picklistValue.multilingualOtherText?has_content and picklistValue.multilingualRemarks?has_contentdoes not return true when defaultText is empty and localizedText has content,
		thus, we don't use that check here nor in printMultilingualPicklist see https://dev.azure.com/echa-ecm/IUCLID/_workitems/edit/633648 -->
			<@printMultilingualField picklistValue.multilingualOtherText false printNA/>
		</#if>
    <#elseif picklistValue?has_content>
        ${picklistValue}
	<#elseif printNA>
		NA
	</#if>
	<#if (printRemarks)>
		<@printMultilingualField picklistValue.multilingualRemarks true false/>
	</#if>
	<#lt>
</#macro>


<#macro printMultilingualPicklistMultiple picklistMultipleValue separator=";" printRemarks=false printNA=true>
	<#if picklistMultipleValue?has_content>
		<#list picklistMultipleValue as item>
			<@printMultilingualPicklist item printRemarks printNA/>
			<#if item_has_next>${separator} </#if>
		</#list>
	<#else>
	NA
	</#if>
</#macro>

<#macro inventoryECNumber inventoryBlock>
<#compress>
	<#if inventoryBlock?has_content>
		<#list inventoryBlock as item>
			<#if item.inventoryCode == 'EC'>
				<@com.text item.numberInInventory/>
			</#if>
		</#list>
	</#if>
</#compress>
</#macro>

<#function isClassificationSpecificTargetOrgan item>
	<#if item.Toxicity.HazardCategory?has_content || item.Toxicity.AffectedOrgans?has_content || item.Toxicity.RouteExposure?has_content || item.Toxicity.HazardStatement?has_content>
		<#return true>
	</#if>
	<#return false/>
</#function>

<#function isClassificationSpecificConcLimits record>
	<#list record.Classification.SpecificConcentrations.ConcentrationRange as item>
		<#if record?has_content>
			<#if item.ConcentrationRangeVal?has_content || item.HazardCategories?has_content>
				<#return true>
			</#if>
		</#if>
	</#list>
	<#return false/>
</#function>
<#function isClassificationHealtHazards record healthHazardClassifications>
	<#if record?has_content>
		<#if (healthHazardClassifications?size > 0) || (record.Classification.HealthHazards.ToxicitySingle.Toxicity?size > 0) || isNotEmptyClassificationBlock(record.Classification.HealthHazards.ReproductiveToxicity.Effects) || isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.ReproductiveToxicity.ReproductiveToxicity) || isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.GermCell.GermCell) || isNotEmptyCRMClassificationBlock(record.Classification.HealthHazards.Carcinogenicity.Carcinogenicity) || record.Classification.AdditionalHazard.Classes?has_content || record.Classification.AdditionalHazard.Statements?has_content || isClassificationSpecificConcLimits(record) >
			<#return true/>
		</#if>
	</#if>
	<#return false/>
</#function>

<#function isClassificationEnvironmentalHazards record>
	<#if record?has_content>
		<#if isNotEmptyClassificationBlock(record.Classification.EnvironmentalHazards.AquaticEnvironment.AcuteShortTerm) || isNotEmptyClassificationBlock(record.Classification.EnvironmentalHazards.AquaticEnvironment.LongTerm) ||record.Classification.EnvironmentalHazards.AquaticEnvironment.MFactor.MFactorAcute?has_content ||record.Classification.EnvironmentalHazards.AquaticEnvironment.MFactor.MFactorChronic?has_content || isNotEmptyClassificationBlock(record.Classification.EnvironmentalHazards.OzoneLayer.HazardousOzone)>
			<#return true>
		</#if>
	</#if>
	<#return false/>
</#function>

<#function isNotEmptyClassificationBlock block>
	<#if !(block?has_content)>
		<#return false>
	</#if>
	<#if block.HazardCategory?has_content || block.HazardStatement?has_content >
		<#return true>
	</#if>
	<#return false>
</#function>

<#--  CRM stands for Carcinogenicity repro tox and germ cell mutagenecity  -->
<#function isNotEmptyCRMClassificationBlock block>
	<#if !(block?has_content)>
		<#return false>
	</#if>
	<#if block.HazardCategory?has_content || block.HazardStatement?has_content>
		<#return true>
	</#if>
	<#return false>
</#function>

<#function isNotEmptyConcentrationBlock record>
	<#if !(record?has_content)>
		<#return false>
	</#if>
	<#if record.ConcentrationRangeVal?has_content || record.HazardCategories?has_content>
		<#return true>
	</#if>
	<#return false>
</#function>

<#function getSynonyms l>
	<#local result = []/>
	<#list l.Synonyms as synonym>
		<#local s>
			<#if (synonym.Identifier?has_content)>
				<@com.picklist synonym.Identifier/>
			</#if>
			${synonym.Name}
            <#if (synonym.Remarks?has_content)>
                <@printMultilingualField synonym.Remarks!{} true false/>
			</#if>
		</#local>
		<#local result = [s] + result/>
	</#list>
	<#return (result?reverse)>
</#function>

<#function substanceIdentifiers referenceSubstance>
<#--  Order of representation/priority
IUPAC name
CAS number
CAS name
EC number
Synonyms
Molecular formula  -->
	<#local iupacName><@com.text (referenceSubstance.IupacName)!""/></#local>
	<#if (referenceSubstance.Inventory?has_content)>
		<#local casNumber><@com.text referenceSubstance.Inventory.CASNumber!""/></#local>
		<#local casName><@com.text referenceSubstance.Inventory.CASName!""/></#local>
		<#local ecNumber>
			<#if (referenceSubstance.Inventory.InventoryEntry?has_content)>
				<@inventoryECNumber referenceSubstance.Inventory.InventoryEntry/>
			</#if>
		</#local>
	</#if>
    <#if (referenceSubstance.Synonyms?has_content)>
        <#local synonyms=getSynonyms(referenceSubstance.Synonyms)/>
    </#if>
	<#if (referenceSubstance.MolecularStructuralInfo?has_content)>
			<#local molecularFormula><@com.text referenceSubstance.MolecularStructuralInfo.MolecularFormula!""/></#local>
	</#if>
	<#return {
		"IUPAC": iupacName,
		"CAS no": casNumber!"",
		"CAS": casName!"",
		"EC no": ecNumber!"",
		"Synonyms": synonyms![],
		"Molecular Formula": molecularFormula!""
	}/>
</#function>

<#function isAdditivesImpuritiesPopulated impuritiesAndAdditives>
	<#local additives = impuritiesAndAdditives.Additives.Additives />
	<#if additives?has_content>
		<#return true />
	</#if>

	<#local impurities = impuritiesAndAdditives.Impurities.Impurities />
	<#if impurities?has_content>
		<#return true />
	</#if>

	<#return false>
</#function>

<#function addSortValue components>
	<#local l = []/>
	<#list components as c>
		<#local l = l + [{"sortedComponent": c, "sortValue": getSortValue(c)}]/>
	</#list>
	<#return l/>
</#function>

<#function getSortValue c>
	<#local smallDifference = 0000.1/>
	<#local value = 0/>
	<#if c.TypicalConcentration.lower.value?has_content>
		<#local value = c.TypicalConcentration.lower.value/>
		<@com.range c.TypicalConcentration/>
	<#elseif c.ConcentrationRange.lower.value?has_content>
		<#local value = c.ConcentrationRange.lower.value/>
		<#if (c.ConcentrationRange.lower.qualifier == ">")>
			<#local value = value + smallDifference/>
		</#if>
	<#else>
		<#return 0/>
	</#if>
	<#return value>
</#function>

<#function sortByConcentration components>
	<#return components?sort_by("sortValue")?reverse>
</#function>

<#macro capitalise text>
  <#local len = text?length />
  <#if (len>0)>
     ${text[0]?upper_case + text[1..]}
  <#else>
    ${text}
  </#if>
</#macro>